include("karaskel.lua")

script_name         = "Random fade"
script_description  = "Fade randomly part of a letter"
script_author       = "Kubat"
script_version      = "1.0"

function effect(subs)
  aegisub.progress.task("Retrieving header data...")
  local meta, styles = karaskel.collect_head(subs)
  aegisub.progress.task("Applying effect...")
  local i, ai, maxi, maxai = 1, 1, #subs, #subs       -- Initial number of lines to process
  while i <= maxi do                                  -- For each line in the subs
    aegisub.progress.task(string.format("Applying effect (%d/%d)...", ai, maxai))
    aegisub.progress.set((ai - 1) / maxai * 100)
    local l = subs[i]
    if l.class == "dialogue" and not l.comment then   -- If the line is a non-comment non-header line
      karaskel.preproc_line(subs, meta, styles, l)    -- Run karaskel preprocessing
      apply_rfade(subs, meta, l)                      -- Apply the complex move transform (THIS IS THE DROID YOU'RE LOOKING FOR)
      subs.delete(i)                                  -- Delete this line (or comment it instead if you prefer)
      maxi = maxi - 1                                 -- We deleted a line, so the loop upper bound must be decreased (to avoid endless loops)
    else
      i = i + 1                                       -- Nothing to see, skip to next line
    end
    ai = ai + 1
  end
  aegisub.progress.task("Finished!")
  aegisub.progress.set(100)
  aegisub.set_undo_point("Complex movement")          -- Set undo point (for Aegisub) after the task is completed.
end

-- Transform the \rfade into a sequence of \fade and \clip
function apply_rfade(subs, meta, line)
  local l     = table.copy(line)                      -- Deep copy of the original line
  local rfade = get_rfade_args(l)                     -- Retrieve the \rfade tag with its parameters
  if rfade == nil then
    subs.append(l)                                    -- If no \rfade on that line, leave it unchanged
  else
    -- Retrieve informations about the \rfade
    local w, h            = tonumber(rfade["w"]), tonumber(rfade["h"])
    local stepx, stepy    = tonumber(rfade["stepx"]), tonumber(rfade["stepy"])
    local T, L            = tonumber(rfade["T"]), tonumber(rfade["L"])
    local fadein, fadeout = tonumber(rfade["fadein"]), tonumber(rfade["fadeout"])
    -- Create the new 'w * h' lines
    for x = 0, (w - 1) do for y = 0, (h - 1) do
      local new_effect = gen_rfade_partial_effect(L + x * stepx, T + y * stepy, stepx, stepy, fadein, fadeout)
      l.text = line.text
      l.text, nb = string.gsub(l.text, "\\rfade%([%d%.]+,[%d%.]+,[%d%.]+,[%d%.]+,[%d%.]+,[%d%.]+,[%d%.]+,[%d%.]+%)", new_effect)
      subs.append(l)
    end end
    -- End of the loops
  end
end

-- Generate a fade of a part of a letter according to the @fadein @fadeout values and the coordinates of the TopLeft corner of the letter's part
function gen_rfade_partial_effect(L, T, stepx, stepy, fadein, fadeout)
  return string.format("\\clip(%f,%f,%f,%f)\\fade(%f,%f)",L, T, L + stepx, T + stepy, math.random(0, fadein), math.random(0, fadeout))
end

-- Retrieve parameters of the \rfade tag in a table
function get_rfade_args(line)
  local rfade_tag = string.match(line.text, "\\rfade%([%d%.]+,[%d%.]+,[%d%.]+,[%d%.]+,[%d%.]+,[%d%.]+,[%d%.]+,[%d%.]+%)")
  if rfade_tag == nil then return nil else
    local rfade = {}
    rfade["fadein"], rfade["fadeout"], rfade["w"], rfade["h"], rfade["T"], rfade["L"], rfade["stepx"], rfade["stepy"] = string.match(rfade_tag, "\\rfade%(([%d%.]+),([%d%.]+),([%d%.]+),([%d%.]+),([%d%.]+),([%d%.]+),([%d%.]+),([%d%.]+)%)")
    return rfade
  end
end

aegisub.register_macro("Random fade", "Create a partial random fade of letters", effect)
